# Registry of identities

## Documentation
[rubydoc.info](http://www.rubydoc.info/gems/soar-registry-identity/)

## Quickstart

### Example data

```javascript
identities = [{
    "uuid": "62936e70-1815-439b-bf89-8492855a7e6b",
    "email": "test+publisher@hetzner.co.za",
}]
 
```

### Directory
Create a directory provider

```ruby
> require 'soar/registry/directory'
> directory_provider = Soar::Registry::Directory::Provider::Stub.new(
    table: "identities",
    index: ["uuid", "email"]
)
```

Create a directory

```ruby
> directory = Soar::Registry::Directory.new(directory_provider)
```

### Instantiation

#### Manual instantiation

##### Staff Email IDR
Search for identifiers by email address. Used by [soar-authentication-identity_uuid_translator](https://github.com/hetznerZA/soar-authentication-identity_uuid_translator) to translate an authenticated identifier to an UUID.
```ruby
require 'soar/registry/identity'
identity_provider = Soar::Registry::Identity::Provider::Staff::Email.new({
    directory: directory,
    fetch_index: 'uuid',
    search_index: 'email'
})
```

```ruby
@email_idr = Soar::Registry::Identity.new(identity_provider)
```

#### Factory instantiation

Create a selector

```ruby
> require 'object_selector'
> selector = ObjectSelector.new(
  ObjectSelector::Provider::RegexRuleList.new({
    rules: [
      {
        regex: /\A[\w+\-.]+@hetzner.co.za\z/i,
        object: Soar::Registry::Identity::Provider::Staff::Email.new({
          directory: Object.new,
          fetch_index: SecureRandom.hex,
          search_index: SecureRandom.hex
        })
      },
      {
        regex: /\A[\w+\-.]+@.+/i,
        object: Soar::Registry::Identity::Provider::Customer::Email.new({
          directory: Object.new,
          fetch_index: SecureRandom.hex,
          search_index: SecureRandom.hex
        })
      },
      {
        regex: /\A[CF]{0,1}\d+\z/,
        object: Soar::Registry::Identity::Provider::Customer::ClientNumber.new({
          directory: Object.new,
          fetch_index: SecureRandom.hex,
          search_index: SecureRandom.hex
        })
      }
    ]
  })
)
```

Get an IDR

```ruby
> selector_value = 'your-string-here'
idr = Soar::Registry::Identity::Factory.create({
  value: selector_value,
  selector: selector
})
```

### Use your IDR

Getting a list of identifiers

```ruby
> identifiers = @email_idr.get_identifiers("admin@hetzner.co.za") 
> puts identifiers.inspect
["admin@hetzner.co.za", "identity-820d5660-2204-4f7d-8c04-746313439b81"]
```

Getting a list of roles

```ruby
> roles = @uuid_idr.get_roles("identity-820d5660-2204-4f7d-8c04-746313439b81")
> # get_roles is not applicable to staff email idr
> puts roles.inspect
["staff", "configuration_publisher", "configuration_consumer"]
```

Getting a hash of attributes for a role 

```ruby
> role = 'staff'
> attributes = @uuid_idr.get_attributes("identity-820d5660-2204-4f7d-8c04-746313439b81", role)
> # get_attributes is not applicable to staff email idr
> puts attributes.inspect
{
    "staff": {
        "department": "technical"
    }
}

```

Getting a hash of all attributes

```ruby
> attributes = @uuid_idr.get_attributes("identity-820d5660-2204-4f7d-8c04-746313439b81")
> # get_attributes is not applicate to staff email idr
> puts attributes.inspect
{
    "identity_id" => "identity-820d5660-2204-4f7d-8c04-746313439b81",
    "entity_id"=> "entity-bad85eb9-0713-4da7-8d36-07a8e4b00eab",
    "email"=> "admin@hetzner.co.za",
    "roles"=> {
        "staff"=> {},
        "configuration_publisher"=> {
            "configuration_identifiers"=> ["*"]
        },
        "configuration_consumer"=> {
            "configuration_identifiers"=> ["*"]
        }
    },
    "address"=> {
        "detail"=> "Belvedere Office Park, Unit F",
        "street"=> "Bella Rosa Street",
        "suburb"=> "Tygervalley",
        "city"=> "Durbanville",
        "postal"=> "7550"
    }
}
```

## Tests

### Local

Start container dependencies

```bash
$ docker-compose up --build --remove-orphans --force-recreate
```

Soar::Registry::Identity::Provider::Customer::Uuid

```bash
$ CUSTOMER_DIRECTORY_CONFIG_FILE=config.mysql.yml ROLES_DIRECTORY_CONFIG_FILE=config.dynamo_db.yml TEST_ORCHESTRATION_PROVIDER=Customer::Uuid bundle exec cucumber --tags ~@future
```

Soar::Registry::Identity::Provider::Staff::Uuid

```bash
$ STAFF_DIRECTORY_CONFIG_FILE=config.ldap.yml ROLES_DIRECTORY_CONFIG_FILE=config.dynamo_db.yml TEST_ORCHESTRATION_PROVIDER=Staff::Uuid bundle exec cucumber 
```

Soar::Registry::Identity::Provider::Stub::Uuid

```bash
$ TEST_ORCHESTRATION_PROVIDER=Stub::Uuid bundle exec cucumber
```

Soar::Registry::Identity::Provider::Staff::Email

```bash
$ bundle exec rspec spec/staff/email_spec.rb
```

Soar::Registry::Identity::Provider::Customer::Email

```bash
$ bundle exec rspec spec/customer/email_spec.rb
```

Soar::Registry::Identity::Provider::Customer::ClientNumber

```bash
$ bundle exec rspec spec/customer/client_number_spec.rb
```

Soar::Registry::Identity::Factory 

```bash
$ bundle exec rspec spec/authenticated_identity_factory_spec.rb spec/identity_uuid_factory_spec.rb
```

### CI

Soar::Registry::Identity::Provider::Stub::Uuid

```bash
docker-compose --file docker-compose.ci.stub-uuid.yml --project-name soar-registry-identity-provider-stub-uuid up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityproviderstubuuid_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.staff-email.yml --project-name soar-registry-identity-provider-stub-uuid down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Provider::Staff::Email

```bash
docker-compose --file docker-compose.ci.staff-email.yml --project-name soar-registry-identity-provider-staff-email up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityproviderstaffemail_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.staff-email.yml --project-name soar-registry-identity-provider-staff-email down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Provider::Customer::Email

```bash
docker-compose --file docker-compose.ci.customer-email.yml --project-name soar-registry-identity-provider-customer-email up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityprovidercustomeremail_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.customer-email.yml --project-name soar-registry-identity-provider-customer-email down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Provider::Customer::ClientNumber

```bash
docker-compose --file docker-compose.ci.customer-client_number.yml --project-name soar-registry-identity-provider-customer-client_number up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityprovidercustomerclientnumber_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.customer-client_number.yml --project-name soar-registry-identity-provider-customer-client_number down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Factory 

```bash
docker-compose --file docker-compose.ci.factory.yml --project-name soar-registry-identity-factory up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityfactory_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.factory.yml --project-name soar-registry-identity-factory down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Provider::Staff::Uuid

```bash
docker-compose --file docker-compose.ci.staff-uuid.yml --project-name soar-registry-identity-provider-staff-uuid up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityproviderstaffuuid_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.staff-uuid.yml --project-name soar-registry-identity-provider-staff-uuid down --rmi local
exit $EXIT_CODE;
```

Soar::Registry::Identity::Provider::Customer::Uuid

```bash
docker-compose --file docker-compose.ci.customer-uuid.yml --project-name soar-registry-identity-provider-customer-uuid up --abort-on-container-exit --remove-orphans --build --force-recreate
EXIT_CODE=$(docker ps -a -f "name=soarregistryidentityprovidercustomeruuid_tests" -q | xargs docker inspect -f "{{ .State.ExitCode }}");
docker-compose --file docker-compose.ci.customer-uuid.yml --project-name soar-registry-identity-provider-customer-uuid down --rmi local
exit $EXIT_CODE;
```

## Resources
* [DynamoDBLocal](http://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DynamoDBLocal.html)
* [Multiple AWS Credentials](https://blogs.aws.amazon.com/security/post/Tx3D6U6WSFGOK2H/A-New-and-Standardized-Way-to-Manage-Credentials-in-the-AWS-SDKs)
* [AWS SDK for Ruby](http://docs.aws.amazon.com/amazondynamodb/latest/gettingstartedguide/GettingStarted.Ruby.html)

## References
* [soar idm](https://github.hetzner.co.za/hetznerZA/soar_idm/blob/master/lib/soar_idm/soar_idm.rb)
* [Domain analysis](https://docs.google.com/a/hetzner.co.za/drawings/d/1vGdzjKPD3gzn1e0bsC4liFCyxY31Qjjxe3y41beVBzw/edit?usp=sharing)
* [staff idr](https://github.hetzner.co.za/hetznerZA/idr_staff/blob/master/idr_staff/lib/idr_staff/staff_idr.rb)
